package cs3500.expenses;

import java.util.Objects;

/**
 * An expense for room and board. Assumes board is bought at a rated price over
 * some number of days.
 * DO NOT EDIT THIS CODE EXCEPT TO ADD METHODS MENTIONED IN THE ASSIGNMENT!!
 */
public final class BoardExpense implements Expense {

  private final String name;
  private final double totalCost;
  private final double rate;
  private final int duration;

  /**
   * Creates a new expense with the given name and costs.
   * Note the total cost is (rate * duration) + taxed amount.
   * @param name name of the expense
   * @param rate cost of the room per duration unit (e.g. night, hour, etc)
   * @param duration amount of duration unit spent in boarding
   * @param taxedAmount amount spent in tax
   * @throws IllegalArgumentException if name is null or empty
   * @throws IllegalArgumentException if rate is negative
   * @throws IllegalArgumentException if duration is not positive
   * @throws IllegalArgumentException if taxedAmount is negative
   */
  public BoardExpense(String name, double rate, int duration, double taxedAmount) {
    if (name == null || name.isEmpty()) {
      throw new IllegalArgumentException("Invalid name given to expense");
    }

    if (rate < 0 || duration <= 0 || taxedAmount < 0) {
      throw new IllegalArgumentException("Invalid costs given to expense");
    }

    this.name = name;
    this.rate = rate;
    this.duration = duration;
    this.totalCost = this.subtotal() + taxedAmount;
  }

  @Override
  public String name() {
    return this.name;
  }

  @Override
  public double totalCost() {
    return this.totalCost;
  }

  @Override
  public double subtotal() {
    return this.rate * duration;
  }

  @Override
  public boolean equals(Object other) {
    if (other == this) {
      return true;
    }

    if (!(other instanceof BoardExpense)) {
      return false;
    }

    BoardExpense that = (BoardExpense) other;
    return this.name.equals(that.name);
  }

  @Override
  public int hashCode() {
    return Objects.hash(this.name);
  }
}
