package cs3500.expenses;

import java.util.ArrayList;
import java.util.List;

/**
 * Expense report that calculates reimbursement as the cost of all expenses
 * contained within minus taxes.
 * DO NOT EDIT THIS CODE EXCEPT TO ADD METHODS MENTIONED IN THE ASSIGNMENT!!
 */
public final class TaxExemptExpenseReport implements ExpenseReport {

  private final List<Expense> expensesToReimburse;

  /**
   * Creates a report with no expenses.
   */
  public TaxExemptExpenseReport() {
    this.expensesToReimburse = new ArrayList<>();
  }

  @Override
  public void addExpense(Expense expense) {
    if (expense == null) {
      throw new IllegalArgumentException("Cannot add null expense");
    }

    if (!this.expensesToReimburse.contains(expense)) {
      this.expensesToReimburse.add(expense);
    } else {
      throw new IllegalArgumentException("Cannot add same expense twice");
    }
  }

  /**
   * Returns the total cost of expenses without their tax included. This is how
   * this form calculates the reimbursement (the amount to be repaid).
   * NOTE: we had to add Javadocs to this method because the implementation does
   * something more specific than what the interface claims.
   * @return the reimbursement for the contained expenses
   */
  @Override
  public double totalReimbursement() {
    double answer = 0.0;
    for (Expense expense : expensesToReimburse) {
      answer += expense.subtotal();
    }
    return answer;
  }

  @Override
  public double totalCost() {
    double answer = 0.0;
    for (Expense expense : expensesToReimburse) {
      answer += expense.totalCost();
    }
    return answer;
  }

  @Override
  public Expense findItemByName(String name) {
    for (Expense expense : expensesToReimburse) {
      if (expense.name().equals(name)) {
        return expense;
      }
    }
    throw new IllegalStateException("No such expense called \"" + name + "\"");
  }
}
