package cs3500.expenses;

import java.util.Objects;

/**
 * An expense for any kind of travel. Assumes travel is reported as
 * cost instead of a rate over some amount of time.
 * DO NOT EDIT THIS CODE EXCEPT TO ADD METHODS MENTIONED IN THE ASSIGNMENT!!
 */
public final class TravelExpense implements Expense {

  private final String name;
  private final double totalCost;
  private final double subtotal;

  /**
   * Creates a travel expense with the associated name and costs.
   * @param name name of the expense
   * @param totalCost total dollar cost of expense with tax
   * @param taxedAmount dollar amount paid in tax
   * @throws IllegalArgumentException if name is null or empty
   * @throws IllegalArgumentException if totalCost is negative
   * @throws IllegalArgumentException if taxedAmount is negative
   */
  public TravelExpense(String name, double totalCost, double taxedAmount) {
    if (name == null || name.isEmpty()) {
      throw new IllegalArgumentException("Invalid name given to expense");
    }

    if (totalCost < 0 || taxedAmount < 0) {
      throw new IllegalArgumentException("Invalid costs given to expense");
    }

    this.name = name;
    this.totalCost = totalCost;
    this.subtotal = totalCost - taxedAmount;
  }

  @Override
  public String name() {
    return this.name;
  }

  @Override
  public double totalCost() {
    return this.totalCost;
  }

  @Override
  public double subtotal() {
    return this.subtotal;
  }

  @Override
  public boolean equals(Object other) {
    if (other == this) {
      return true;
    }

    if (!(other instanceof TravelExpense)) {
      return false;
    }

    TravelExpense that = (TravelExpense) other;
    return this.name.equals(that.name);
  }

  @Override
  public int hashCode() {
    return Objects.hash(name);
  }
}
